/******************************************************************************/
/*  (C) COPYRIGHT International Business Machines Corp. 1993                  */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THIS MATERIAL        */
/*  "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,        */
/*  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY  */
/*  AND FITNESS FOR A PARTICULAR PURPOSE.                                     */
/*                                                                            */
/******************************************************************************/

#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXExtlib.h>
#include <misc.h>
#include <math.h>

#define BOGUS_TYPE 100

/****************************************************************************
 *  Tests circular and elliptical arcs with boundary conditions             *
 ***************************************************************************/
void draw_arc(Display          *dpy,
              XID              resourceID,
              PEXOCRequestType req_type)

  {
    PEXCoord    center, pts2[2];
    PEXCoord2D  center_2;
    PEXVector   major, minor;
    PEXVector2D major_2, minor_2;
    PEXColor    line_color;
    double      radius;

    /*****************************************************************
     Top Row. EllipticalArc3D with different line attributes
    *****************************************************************/

    center.x = 0.20;
    center.y = 0.8;
    center.z = 0.0;

    major.x = 0.15;
    major.y = 0.0;
    major.z = 0.0;

    minor.x = 0.0;
    minor.y = 0.0;
    minor.z = 0.0;

    PEXExtEllipticalArc(dpy, resourceID, req_type, &center, &major, &minor,
      M_PI_2, 2.0*M_PI);

    center.x = 0.55;
    minor.z = 0.1;                /* Minor along the Z-axis */
    PEXSetLineType (dpy, resourceID, req_type, PEXLineTypeDashed);
    PEXExtEllipticalArc(dpy, resourceID, req_type, &center, &major, &minor,
      0.0, M_PI);              /* Horizontal dashed line */

    line_color.rgb.red = 1.0;
    line_color.rgb.green = 0.0;
    line_color.rgb.blue = 0.0;
    PEXSetLineColor(dpy, resourceID, req_type, PEXColorTypeRGB, &line_color);
    center.x = 0.79;
    major.x = 0.0;
    minor.y = 0.15;
    minor.z = 0.0;                 /* Vertical dashed line */
    PEXExtEllipticalArc(dpy, resourceID, req_type, &center, &major, &minor,
      0.0, -M_PI);

    minor.y = 0.0;                 /* Major and Minor = 0 */
    line_color.indexed.index = 6;
    PEXSetLineColor(dpy, resourceID, req_type, PEXColorTypeIndexed, 
     &line_color);
    PEXSetLineType (dpy, resourceID, req_type, PEXLineTypeDashDot);
    PEXSetLineWidth(dpy, resourceID, req_type, 3.0);
    center.x = 0.92;
    PEXExtEllipticalArc(dpy, resourceID, req_type, &center, &major, &minor,
      0.0, 2.0*M_PI);

    /******************************************************************
     * Middle Row: Circular arcs                                      *
     *****************************************************************/
    line_color.indexed.index = 1;
    PEXSetLineColor(dpy, resourceID, req_type, PEXColorTypeIndexed,
     &line_color);
    PEXSetLineType (dpy, resourceID, req_type, BOGUS_TYPE);
    PEXSetLineWidth(dpy, resourceID, req_type, 1.0);

    center_2.x = 0.17;
    center_2.y = 0.5;
    radius = 0.14;
    PEXExtCircularArc2D(dpy, resourceID, req_type, &center_2, radius,
      M_PI, 2.0*M_PI);

    PEXSetIndividualASF(dpy, resourceID, req_type, PEXASFLineColor,
     PEXBundled);
    PEXSetLineBundleIndex(dpy, resourceID, req_type, 2);
    PEXSetLineType (dpy, resourceID, req_type, PEXLineTypeDashed);
    center_2.x = 0.5;
    PEXExtCircularArc2D(dpy, resourceID, req_type, &center_2, radius,
     0.0, 2.0*M_PI);            /* Complete Circle */

    radius = 0.0;               /* Rendered as apoint */
    center_2.x = 0.83;
    PEXSetLineType (dpy, resourceID, req_type, PEXLineTypeSolid);
    PEXExtCircularArc2D(dpy, resourceID, req_type, &center_2, radius,
     0.0, M_PI);

    /*****************************************************************
     Bottom Row. EllipticalArc2D with different line attributes
    *****************************************************************/
    
    center_2.x = 0.20;             /* Horizontal line */
    center_2.y = 0.2;
    major_2.x = 0.15;
    major_2.y = 0.0;
    minor_2.x = 0.0;
    minor_2.y = 0.0;
    PEXExtEllipticalArc2D(dpy, resourceID, req_type, &center_2, &major_2, 
     &minor_2, 0.0, M_PI);

    PEXSetIndividualASF(dpy, resourceID, req_type, PEXASFLineColor,
     PEXIndividual);
    PEXSetLineWidth(dpy, resourceID, req_type, 3.0);
    line_color.rgb.red = 0.0;
    line_color.rgb.green = 1.0;
    line_color.rgb.blue = 0.0;
    PEXSetLineColor(dpy, resourceID, req_type, PEXColorTypeRGB, &line_color);
    center_2.x = 0.55;
    PEXExtEllipticalArc2D(dpy, resourceID, req_type, &center_2, &major_2, 
     &minor_2, M_PI_2, M_PI);
    PEXSetLineWidth(dpy, resourceID, req_type, 1.0);
    PEXExtEllipticalArc2D(dpy, resourceID, req_type, &center_2, &major_2, 
     &minor_2, 3.0*M_PI_2, 2.0*M_PI);

    center_2.x = 0.8;               /* Vertical line */
    major_2.x = 0.0;
    minor_2.y = 0.1;
    PEXExtEllipticalArc2D(dpy, resourceID, req_type, &center_2, &major_2, 
     &minor_2, 0.0, 2.0*M_PI);

    center.x = 0.92;                /* Rendered as a point */
    minor_2.y = 0.0;
    PEXExtEllipticalArc2D(dpy, resourceID, req_type, &center_2, &major_2, 
      &minor_2, 0.0, M_PI);

  }

/****************************************************************************
 *  Sets the test parameters                                                *
 ***************************************************************************/
void inquire_test_params(char *test_name,
                         int  *num_images,
                         int  *supported_modes,
                         char *win_title)


  {
    strcpy(test_name, "arc_bounds");
    *num_images = 1;
    *supported_modes = IMM_MODE | STR_MODE;

    strcpy(win_title, "Arc_bounds");
  }


/****************************************************************************
 *  This routine is used to do miscellaneous things like setting some       *
 *  renderer attributess, Viewing etc if needed.                            *
 ***************************************************************************/
void misc_setup(Display *dpy, Window window, PEXRenderer renderer,
                int cur_image)
  {
  }

/****************************************************************************
 *  This routine calls the function which does the actual test              *
 ***************************************************************************/
void execute_test(Display *dpy, XID resourceID,
               PEXOCRequestType req_type, int cur_image)
  {

    describe_test( "\n" );
    describe_test( " Tests Arcs with boundary conditions:\n");
    describe_test( "\n" );
    describe_test( " TOP ROW:4 elliptical3D arcs. 1st: minor = 0\n");
    describe_test( "         Should see a  White Horizontal line.\n");
    describe_test( "         2nd: Minor along Z-axis. Vertical Line\n");
    describe_test( "         3rd: major = 0, red vertical dashed lines.\n");
    describe_test( "         4th: Major,Minor = 0, should be rendered as point.\n");
    describe_test( " MIDDLE ROW:3 Circular Arcs. \n");
    describe_test( "            1st: Uses unsupp. line type & should be drawn with default type.\n");
    describe_test( "            2nd: Complete circular arc using bundled line color.\n");
    describe_test( "            3rd: Radius = 0, should be rendered as a point.\n");
    describe_test( " BOTTOM ROW:5 elliptical2D arcs.\n");
    describe_test( "            1st: Minor = 0, Should see a yellow horizontal line.\n");
    describe_test( "            2nd: Thick green line. 3rd: Thin green line.\n");
    describe_test( "            4th: Major = 0, Vertical line.\n");
    describe_test( "            5th: Major, Minor = 0, should be rendered as a point.\n");
    describe_test( "\n"); 

    draw_arc(dpy, resourceID, req_type);

  }

void testproc(Display *dpy, Window window, int cur_image)
  {
    image_testproc("arc_bounds", "arc_bounds", img_desc,
                    dpy,window, 0, 0,
                    glob_window_width, glob_window_height);
  }

