/******************************************************************************/
/*  (C) COPYRIGHT International Business Machines Corp. 1993                  */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THIS MATERIAL        */
/*  "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,        */
/*  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY  */
/*  AND FITNESS FOR A PARTICULAR PURPOSE.                                     */
/*                                                                            */
/******************************************************************************/
#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXExtlib.h>
#include <misc.h>

Window local_window;
/****************************************************************************
 *   No primitives drawn in this test                                       *
 ***************************************************************************/
void draw_model(Display          *dpy,
               XID              resourceID,
               PEXOCRequestType req_type)
{
   Drawable drawable;
   PEXColorApproxEntry *cptr, *return_capx;
   PEXPointer ptr;
   unsigned long count, mask[2];
   PEXExtRendererAttributes *rcv_attrs;    
   int all, ret_val, def_act, table_type, i; 

   drawable = local_window;

   mask[0] = 0;
   mask[1] = 0;
   PEXExtSetRendererAttributeMask(mask, PEXRAColorApproxTable);
   rcv_attrs = PEXExtGetRendererAttributes(dpy, resourceID, mask);

   ptr = PEXGetTableEntry(dpy, rcv_attrs->color_approx_table, 0, 
                 PEXRealizedValue, &def_act, &table_type);
   
   cptr = (PEXColorApproxEntry*)ptr; 
   ret_val = PEXExtQueryColorApprox(dpy, drawable, cptr, &count, 
                     &return_capx, &all); 
  
   if (ret_val) 
     _ibmpex_stderr_print("Color Approx is supported!\n");
   else 
     _ibmpex_stderr_print("ERROR: Color Approx is NOT supported!\n");  
   
   /* Change the color approximation to an invalid value.  This should 
      cause the PEXExtQueryColorApprox routine to return a list of 
      valid color approx entries.
    
      All weights of the PEXColorRange approx type will be left at 0 
      indicating an invalid color approx type.                        */

   cptr->type = PEXColorRange; 

   ret_val = PEXExtQueryColorApprox(dpy, drawable, cptr, &count,
                     &return_capx, &all); 
  
   if (ret_val) 
     _ibmpex_stderr_print("ERROR: Color Approx should not be supported!\n");
   else {
     _ibmpex_stderr_print("Color Approx is NOT supported!\n\n");  
     _ibmpex_stderr_print("List of Color Approximations returned:\n\n");
     for(i=0; i<count; i++) {
         _ibmpex_stderr_print("Index %d\n",i);
         if (return_capx[i].type == PEXColorSpace) 
	   _ibmpex_stderr_print("Type = PEXColorSpace\n");
         else if (return_capx[i].type == PEXColorRange)
	   _ibmpex_stderr_print("Type = PEXColorRange\n");
         else
	   _ibmpex_stderr_print("Type = %d\n",return_capx[i].type);

         if (return_capx[i].model == PEXColorApproxRGB) 
	   _ibmpex_stderr_print("Model = PEXColorApproxRGB\n");
         else
	   _ibmpex_stderr_print("Model = %d\n",return_capx[i].model);
         
         _ibmpex_stderr_print("max1 = %d, max2 = %d, max3 = %d\n", 
                  return_capx[i].max1, return_capx[i].max2, 
                  return_capx[i].max3);
         if (return_capx[i].dither) _ibmpex_stderr_print("Dither = ON\n");
                  else _ibmpex_stderr_print("Dither = OFF\n");
         _ibmpex_stderr_print("mult1 = %u, mult2 = %u, mult3 = %u\n",
                  return_capx[i].mult1, return_capx[i].mult2, 
                  return_capx[i].mult3);
         _ibmpex_stderr_print("weight1 = %f, weight2 = %f, weight3 = %f\n",
                  return_capx[i].weight1, return_capx[i].weight2,
                  return_capx[i].weight3);
         _ibmpex_stderr_print("base_pixel = %u\n\n", 
                  return_capx[i].base_pixel);
     }
   }
}

/****************************************************************************
 *  Sets the test parameters                                                *
 ***************************************************************************/
void inquire_test_params(char *test_name,
                         int  *num_images,
                         int  *supported_modes,
                         char *win_title)


{
    strcpy(test_name, "Query Color Approx Test");
    *num_images = 1;
    *supported_modes = IMM_MODE | STR_MODE;

    strcpy(win_title, "PEX Query Color Approx Test");
}


/****************************************************************************
 *  This routine is used to do miscellaneous things like setting some       *
 *  renderer attributess, Viewing etc if needed.                            *
 ***************************************************************************/
void misc_setup(Display *dpy, Window window, PEXRenderer renderer,
                int cur_image)
{
   local_window = window;
   _ibmpex_set_stderr("getcolor", "getcolor");
}

/****************************************************************************
 *  This routine calls the function which does the actual test              *
 ***************************************************************************/
void execute_test(Display *dpy, XID resourceID,
               PEXOCRequestType req_type, int cur_image)
{
    describe_test( "\n" );
    describe_test( " IBM Query Color Approx Test:\n");
    describe_test( "\n" );
    describe_test( " This test will perform the following:\n");
    describe_test( "\n"); 
    describe_test( "     1) Perform a PEXExtQueryColorApprox with a valid\n"); 
    describe_test( "        color approximation.\n");
    describe_test( "     2) Perform a PEXExtQueryColorApprox with an invalid\n"); 
    describe_test( "        color approx.  This should cause the routine to \n");
    describe_test( "        return a list of valid color approximations.\n");

    draw_model(dpy, resourceID, req_type);

}

void testproc(Display *dpy, Window window, int cur_image)
{
    file_testproc("getcolor.txt", "getcolor");
}

